<?php
namespace Geral\Repositories;

class Itrequesto extends \Doctrine\ORM\EntityRepository
{
    public function consultaItensRequisicao($filial, $requisicao, $organizacao)
    {
        $sql = <<<SQL
              SELECT IT.CDFILIAL,
                F.NMFILIAL,
                IT.NRREQUESTO,
                IT.NRLANCESTQ,
                IT.NRSEQUITEM,
                IT.CDPRODUTO,
                PR.CDARVPROD,
                PR.NMPRODUTO,
                PR.SGUNIDADE,
                PR.VRFATOCONV,
                IT.QTREQUESTO,
                IT.QTDEVOREQU,
                IT.IDSTITREQU,
                IT.CDFILILANC     AS "CDFILIMOVI",
                FI.NMFILIAL       AS "NMFILIMOVI",
                IT.NRLOTEESTQ,
                IT.NRSUBLOTE,
                PR.CDPRODESTO     AS "CDPRODMOVI",
                RE.CDALMOXSAID,
                A.DSALMOXARIFE    AS "NMALMOXARIFE",
                RE.CDLCESTSAID,
                L.DSLOCALESTOQ    AS "NMLOCALESTOQ",
                IT.NRORG          AS "NRORG",
                IT.DTINCLUSAO     AS "DTINCLUSAO",
                IT.DTULTATU       AS "DTULTATU",
                IT.NRORGINCLUSAO  AS "NRORGINCLUSAO",
                IT.NRORGULTATU    AS "NRORGULTATU",
                IT.CDOPERINCLUSAO AS "CDOPERINCLUSAO",
                IT.CDOPERULTATU   AS "CDOPERULTATU"
              FROM ITREQUESTO IT, PRODUTO PR,
                REQUESTO RE, FILIAL F, FILIAL FI,
                ALMOXARIFADO A, LOCALESTOQUE L
              WHERE IT.CDFILIAL               = :P_CDFILIAL
                AND IT.NRREQUESTO             = :P_REQUISICAO
                AND IT.NRORG                  = :P_NRORG
                AND IT.CDPRODUTO              = PR.CDPRODUTO
                AND IT.CDFILIAL               = RE.CDFILIAL
                AND IT.CDFILIAL               = F.CDFILIAL(+)
                AND IT.CDFILIAL               = FI.CDFILIAL(+)
                AND IT.IDSTITREQU             = NVL('', IDSTITREQU)
                AND IT.NRREQUESTO             = RE.NRREQUESTO
                AND IT.CDFILIAL               = A.CDFILIAL
                AND NVL(RE.CDALMOXSAID, ' ')  = A.CDALMOXARIFE(+)
                AND RE.CDFILIAL               = L.CDFILIAL(+)
                AND NVL(RE.CDALMOXSAID, ' ')  = L.CDALMOXARIFE(+)
                AND NVL(RE.CDLCESTSAID, ' ')  = L.CDLOCALESTOQ(+)
              ORDER BY IT.CDPRODUTO
SQL;
        $params = array(
            "P_CDFILIAL"   => $filial,
            "P_REQUISICAO" => $requisicao,
            "P_NRORG"      => $organizacao,
        );
        try{
            return self::getEntityManager()->getConnection()->fetchAll($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function verificaEstoque(
        $filial, $almoxarifado, $localEstoque, $lote, $sublote, $produto, $dataRequisicao, $requisicao,
        $dataPosicaoEstoque
    )
    {

        $data = self::ultimaDataPosicaoEstoque(
            $filial, $almoxarifado, $localEstoque, $lote, $sublote, $produto , $dataPosicaoEstoque
        );

        $sql = <<<SQL
          SELECT P.CDPRODUTO,  P.NMPRODUTO,  P.SGUNIDADE, IDCNTRESTOQ,
            P.VRFATOCONV, (D.QTESTOQDIA/P.VRFATOCONV) + NVL((QTLANCTOEST/P.VRFATOCONV),0) AS QTESTOQDIA,
            P.CDPRODESTO,
            NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, TO_NUMBER(NVL(CDTIPCUSTEST, '3')), :P_DTCUSTO, SYSDATE,
            D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
            NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, 4, :P_DTCUSTO, SYSDATE,
            D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOBRUT
          FROM PRODUTO P, POSESTDIA D, PRODFILI F, PARAMFILIAL PF,
          (SELECT QTLANCTOEST, CDPRODUTO
            FROM ITLANCTOEST
            WHERE CDPRODMOVI  = :P_CDPRODUTO
              AND CDFILIMOVI  = :P_CDFILIAL
              AND CDFILIAL    = :P_CDFILIPROC
              AND NRLANCESTQ  = :P_NRLANCESTQ) I
          WHERE (P.CDPRODUTO    = :P_CDPRODUTO)
            AND (D.CDFILIAL     = :P_CDFILIAL)
            AND (D.CDALMOXARIFE = :P_CDALMOXARIFE)
            AND (D.CDLOCALESTOQ = :P_CDLOCALESTOQ)
            AND (D.NRLOTEESTQ   = :P_NRLOTEESTQ)
            AND (D.NRSUBLOTE    = :P_NRSUBLOTE)
            AND (D.CDPRODUTO    = P.CDPRODESTO)
            AND (D.CDPRODUTO    = I.CDPRODUTO(+))
            AND (P.CDPRODESTO   = F.CDPRODUTO)
            AND (F.CDFILIAL     = D.CDFILIAL)
            AND (PF.CDFILIAL    = D.CDFILIAL)
            AND (DTPOSIESTQ     = :P_DTPOSICAO)
SQL;
        $params = array(
            "P_CDFILIAL"        => $filial,
            "P_DTCUSTO"         => $dataRequisicao,
            "P_CDPRODUTO"       => $produto,
            "P_CDFILIPROC"      => $filial,
            "P_NRLANCESTQ"      => $requisicao,
            "P_CDALMOXARIFE"    => $almoxarifado,
            "P_CDLOCALESTOQ"    => $localEstoque,
            "P_NRLOTEESTQ"      => $lote,
            "P_NRSUBLOTE"       => $sublote,
            "P_CDPRODESTO"      => $produto,
            "P_DTPOSICAO"       => $data[0],
        );
        try{
            return self::getEntityManager()->getConnection()->fetchAssoc($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function ultimaDataPosicaoEstoque(
        $filial, $almoxarifado, $localEstoque, $lote, $sublote, $produto, $dataPosicaoEstoque
    )
    {
        $data = <<<SQL
          SELECT MAX(DTPOSIESTQ) DTPOSIESTQ
          FROM POSESTDIA
          WHERE (CDFILIAL = :P_CDFILIAL)
            AND (CDALMOXARIFE = :P_CDALMOXARIFE)
            AND (CDLOCALESTOQ = :P_CDLOCALESTOQ)
            AND (NRLOTEESTQ = :P_NRLOTEESTQ)
            AND (NRSUBLOTE = :P_NRSUBLOTE)
            AND (CDPRODUTO = :P_CDPRODESTO)
            AND (DTPOSIESTQ <= :P_DTPOSICAO)
SQL;
        $param = array(
            "P_CDFILIAL"        => $filial,
            "P_CDALMOXARIFE"    => $almoxarifado,
            "P_CDLOCALESTOQ"    => $localEstoque,
            "P_NRLOTEESTQ"      => $lote,
            "P_NRSUBLOTE"       => $sublote,
            "P_CDPRODESTO"      => $produto,
            "P_DTPOSICAO"       => $dataPosicaoEstoque,
        );

        return self::getEntityManager()->getConnection()->fetchArray($data, $param);
    }


    public function executaCustoProduto($filial, $produto, $dataRetirada, $almoxarifado,
        $localEstoque, $lote, $subLote
    )
    {
        $sql = <<<SQL
          SELECT NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO, :P_CDTIPORETI, :P_DATA,
            SYSDATE, :P_ALMOXARIFADO, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE), 0) VALOR FROM DUAL
SQL;
        $params = array(
            "P_CDFILIAL"        => $filial,
            "P_CDPRODUTO"       => $produto,
            "P_CDTIPORETI"      => '3',
            "P_DATA"            => $dataRetirada,
            "P_ALMOXARIFADO"    => $almoxarifado,
            "P_CDLOCALESTOQ"    => $localEstoque,
            "P_NRLOTEESTQ"      => $lote,
            "P_NRSUBLOTE"       => $subLote,
        );

        try{
            return self::getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function consultaSituacaoRequisicao($filial, $requisicao, $organizacao)
    {
        $sql = <<<SQL
          SELECT IDSTITREQU
          FROM ITREQUESTO R
          WHERE  CDFILIAL   = :P_CDFILIAL
           AND NRREQUESTO   = :P_NRREQUESTO
           AND NRORG        = :P_NRORG
          GROUP BY IDSTITREQU
SQL;
        $params = array(
            "P_CDFILIAL"    => $filial,
            "P_NRREQUESTO"  => $requisicao,
            "P_NRORG"       => $organizacao,
        );

        try {
            return self::getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}